#ifndef __MAPPEDREAD_H__
#define __MAPPEDREAD_H__

#include "LocationScore.h"

#include <memory.h>
#include <vector>

// A read with a list of scores fitting the initial predicate
// (i.e., each score is greater than cs_threshhold)
struct MappedRead
{
private:

public:
	int const ReadId;
	volatile int Calculated;
	int TopScore;
	volatile int Lock;
	int EqualScoringID;
	uint EqualScoringCount;
	std::vector<LocationScore*> Scores;
	MappedRead * Paired;
	float Identity;
	char Strand;
	int QStart;
	int QEnd;

	char * Seq;
	
	char * Buffer1;
	char * Buffer2;

	MappedRead(int const readid) :
		ReadId(readid),
		Calculated( 0 ), 
		TopScore( -1 ), 
		Lock( 0 ),
		EqualScoringID( 0 ),
		EqualScoringCount( 1 ),
		Scores(),
		Paired( 0 ),
		Identity( 0 ),
		Strand( '?' ),
		QStart( 0 ),
		QEnd( 0 ),
		Seq( 0 )
	{
		try
		{
			Buffer1 = new char[SequenceProvider.GetReadMaxLen()*4];
			Buffer2 = new char[SequenceProvider.GetReadMaxLen()*4];
			*(int*)Buffer1 = 0x212121;
			*(int*)Buffer2 = 0x212121;
			/*if (Scores == 0)
				throw 0;*/
		}
		catch (...)
		{
			Log.Error("Error allocating MappedRead object (readid=%i)", readid);
		}
	}
	~MappedRead()
	{
		for (size_t i = 0; i < Scores.size(); ++i)
			delete Scores[i];
		delete[] Buffer1;
		delete[] Buffer2;
	}

	// Saves score to this Read and returns a pointer to the saved object
	LocationScore * AddScore(LocationScore score)
	{
		score.Read = this;
		Scores.push_back(new LocationScore(score));
		return Scores[Scores.size()-1];
	}

	LocationScore * TLS() const
	{
		if (TopScore == -1)
			Log.Error("Top score accessed without calculation (Read %i, ESID %i, ESC %i)", ReadId, EqualScoringID, EqualScoringCount);
		return Scores[TopScore];
	}

	int nScores() const
	{
		return Scores.size();
	}
};


#endif
