#ifndef __SW_H__
#define __SW_H__

#include "NGM.h"
#include "NGMThreads.h"

#include "IAlignment.h"

#include <list>
#include "limits.h"

#undef module_name
#define module_name "SW"

class SW : public NGMTask
{
private:
	int m_PairMinDistance;
	int m_PairMaxDistance;
	
	int const m_AlignMode;

	int m_Ivk_mode;
	int m_Ivk_batchSize;
	char const * const * m_Ivk_refSeqList;
	char const * const * m_Ivk_qrySeqList;
	Align * m_Ivk_results;
	int m_Ivk_return;

	void LaunchInvoked();
	void SendToPostprocessing(MappedRead * read);
	void PairedReadSelection(MappedRead * read1, MappedRead * read2);
	bool CheckPairs(LocationScore * ls1, LocationScore * ls2, float & , int &);

	char const * * m_QryBuffer;
	char const * * m_RefBuffer;
	float * m_ScoreBuffer; 

	std::list<MappedRead*> m_ReadBuffer;
	bool CommitReads();
public:
	SW() :
		m_PairMinDistance( (NGM.Paired()) ? Config.GetInt("pair_min_distance", 0, -1) : 0 ),
		m_PairMaxDistance( (NGM.Paired()) ? Config.GetInt("pair_max_distance", -1, -2) : 0 ),
		m_AlignMode( Config.GetInt("sw_mode", 0, 1) )
	{
		NGM.bSWO.Register();

		if (m_PairMaxDistance <= 0)
			m_PairMaxDistance = INT_MAX;

		m_QryBuffer = 0;
		m_RefBuffer = 0;
		m_ScoreBuffer = 0;
	}
	~SW()
	{
		Log.Green("SW dtor");
	}

	void DoRun();
	inline int GetStage() const { return 2; }
	inline char const * GetName() const { return "SW"; }
};

#endif
